create or alter function GetTeamStats
(
	@teamId int,
	@gameDate date,
	@nbrDays int
)
returns table
as
return
with TeamGames as
(
	select
		gd.GameDate,
		g.AwayTeamScore TeamScore,
		case when g.AwayTeamScore > g.HomeTeamScore then 1 else 0 end Wins
	from dbo.Game g
	join dbo.ScheduledGame sg on g.GameId = sg.GameId
	join dbo.GameDay gd on sg.GameDayId = gd.GameDayId
	where sg.AwayTeamId = @teamId
	and sg.SeriesDescription = 'Regular Season'
	and gd.GameDate < @gameDate

	union all

	select
		gd.GameDate,
		g.HomeTeamScore TeamScore,
		case when g.HomeTeamScore > g.AwayTeamScore then 1 else 0 end Wins
	from dbo.Game g
	join dbo.ScheduledGame sg on g.GameId = sg.GameId
	join dbo.GameDay gd on sg.GameDayId = gd.GameDayId
	where sg.HomeTeamId = @teamId
	and sg.SeriesDescription = 'Regular Season'
	and gd.GameDate < @gameDate
), MostRecentGames as
(
	select top(@nbrDays) g.TeamScore, g.Wins
	from TeamGames g
	order by g.GameDate desc
)
select
	avg(cast(g.Wins as float)) AvgWins,
	avg(cast(g.TeamScore as float)) AvgScore
from MostRecentGames g;
go

-- Kansas City Royals stats for the 40 games leading up to 9/1/21
select AvgWins, AvgScore
from MLB.dbo.GetTeamStats(118, '2021-09-01', 40);

select
	gs.AvgWins,
	gs.AvgScore,
	case when sg.AwayTeamId = t.TeamId then g.AwayTeamScore - g.HomeTeamScore else g.HomeTeamScore - g.AwayTeamScore end ScoreDelta
from dbo.Team t
join dbo.ScheduledGame sg on (sg.AwayTeamId = t.TeamId or sg.HomeTeamId = t.TeamId)
join dbo.Game g on sg.GameId = g.GameId
join dbo.GameDay gd on sg.GameDayId = gd.GameDayId
cross apply dbo.GetTeamStats(t.TeamId, gd.GameDate, 40) gs
where t.TeamId not in (159, 160)
and gd.GameDate >= '2021-07-14'
and gd.GameDate <= '2021-12-31'
and sg.SeriesDescription = 'Regular Season'

